/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from 'react';
//imports material ui components
import { Typography, TextField, Button, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import ListRoundedIcon from '@material-ui/icons/ListRounded';
import Chip from '@material-ui/core/Chip';
import { makeStyles } from '@material-ui/core/styles';
import SaveIcon from '@material-ui/icons/Save';
import DeleteRoundedIcon from '@material-ui/icons/DeleteRounded';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import Switch from '@material-ui/core/Switch';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import MenuItem from '@material-ui/core/MenuItem';
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
import { Store } from '../../../../Store';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
//imports services 
import { getRequest, putRequest, deleteRequest, postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { EditFilter, GetAData } from '../../../Filters';
//routers
import { Link, useHistory } from 'react-router-dom';
import Unauthorized from '../Unauthorized';

const useStyles = makeStyles((theme) => ({
    root: {
        display: 'flex',
        justifyContent: 'center',
        flexWrap: 'wrap',
        listStyle: 'none',
        padding: theme.spacing(0.5),
        margin: 0,
    },
    chip: {
        margin: theme.spacing(0.5),
    },
}));

const EditUser = () => {
    const classes = useStyles();
    let history = useHistory();
    const { state } = useContext(Store)
    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("id");

    const [error, setError] = useState(id !== "-1" ? null : false); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(id === "-1" ? true : false); //Necessary to consult the API, wait until complete

    const [switchState, setSwitchState] = useState(id !== "-1" ? false : false)
    const [isLoading, setIsLoading] = useState(false);
    const [name, setName] = useState(id !== "-1" ? "" : "Novo usuário")
    const [email, setEmail] = useState("")
    const [description, setDescription] = useState("")
    const [pass, setPass] = useState("")
    const [confirmPass, setConfirmPass] = useState("")
    const [rolesList, setRolesList] = useState([
        {
            id: 1,
            value: "teacher"
        },
        {
            id: 2,
            value: "student"
        },
        {
            id: 3,
            value: "admin"
        },
        {
            id: 4,
            value: "curator"
        },
        {
            id: 5,
            value: "moderator"
        },
        {
            id: 6,
            value: "supervisor"
        },
        {
            id: 7,
            value: "editor"
        },
        {
            id: 9,
            value: "partner"
        },
        {
            id: 10,
            value: "publisher"
        },
        {
            id: 11,
            value: "submitter"
        },
    ])
    const [userRoles, setUserRoles] = useState([])


    const [errorInName, setErrorInName] = useState({
        error: false,
        message: ''
    })
    const [errorInEmail, setErrorInEmail] = useState({
        error: false,
        message: ''
    })
    const [errorInDesc, setErrorInDesc] = useState({
        error: false,
        message: ''
    })
    const [errorInPass, setErrorInPass] = useState({
        error: false,
        message: ''
    })
    const [errorInConfirmPass, setErrorInConfirmPass] = useState({
        error: false,
        message: ''
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
        setName(e.target.value)
    }

    const EmailHandler = (e) => {
        if (errorInEmail.error) {
            setErrorInEmail({
                error: false,
                message: ''
            })
        }
        setEmail(e.target.value)
    }

    const DescHandler = (e) => {
        if (errorInDesc.error) {
            setErrorInDesc({
                error: false,
                message: ''
            })
        }
        setDescription(e.target.value)
    }

    const PassHandler = (e) => {
        if (errorInPass.error) {
            setErrorInPass({
                error: false,
                message: ''
            })
        }
        setPass(e.target.value)
    }

    const ConfirmPassHandler = (e) => {
        if (errorInConfirmPass.error) {
            setErrorInConfirmPass({
                error: false,
                message: ''
            })
        }
        setConfirmPass(e.target.value)
    }

    // Fields
    const fields = [
        {
            label: 'ID',
            value: id,
            default: true,
            type: 'text'
        },
        {
            label: 'Nome',
            value: name,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event),
            default: false,
            type: 'text',
        },
        {
            label: 'Email',
            value: email,
            error: errorInEmail.error,
            errorMessage: errorInEmail.message,
            onChange: (event) => EmailHandler(event),
            default: false,
            type: 'text'
        },
        {
            label: 'Descrição',
            value: description,
            error: errorInDesc.error,
            errorMessage: errorInDesc.message,
            onChange: (event) => DescHandler(event),
            default: false,
            type: 'text'
        },
        {
            label: 'Senha',
            value: pass,
            error: errorInPass.error,
            errorMessage: errorInPass.message,
            onChange: (event) => PassHandler(event),
            default: id !== "-1" ? true : false,
            type: 'text'
        },
        {
            label: 'Confirmar senha',
            value: confirmPass,
            error: errorInConfirmPass.error,
            errorMessage: errorInConfirmPass.message,
            onChange: (event) => ConfirmPassHandler(event),
            default: id !== "-1" ? true : false,
            type: 'text'
        },
    ]

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const CheckUserPermission = () => {
        let canUserEdit = false;

        if (state.userIsLoggedIn) {
            const roles = [...state.currentUser.roles];
            for (let i = 0; i < roles.length; i++)
                if (roles[i].name === 'admin' || roles[i].name === 'editor')
                    canUserEdit = true;
        }
        else {
            canUserEdit = false;
        }

        return canUserEdit;
    }

    const handleDelete = (chipToDelete) => () => {
        const currRolesList = [...rolesList];
        currRolesList.push({
            id: chipToDelete.id,
            value: chipToDelete.name
        })
        setRolesList(currRolesList)
        setUserRoles((chips) => chips.filter((chip) => chip.id !== chipToDelete.id));
    };

    const deleteUser = (userId) => {
        deleteRequest(
            `/users/${userId}`,
            (data) => {
                if (data.errors)
                    HandleSnack("Erro!", true, "warning", "#FA8072");
                else {
                    HandleSnack(`${name} deletado com sucesso!`, true, "success", "#228B22");
                    history.push("/admin/usersList")
                }
            },
            (error) => {
                HandleSnack("Erro!", true, "warning", "#FA8072");
            }
        )
    }

    const onSubmit = async () => {
        setIsLoading(true)
        let userRolesID = [];
        const api = EditFilter('users', id)
        userRoles.map((role) => (
            userRolesID.push(role.id)
        ))
        const body = {
            "user": {
                "name": name,
                "email": email,
                "description": description,
                "role_ids": userRolesID,
                "terms_of_service": switchState
            }
        }
        putRequest(
            api,
            body,
            (data) => {
                setIsLoading(false)
                if (data.id)
                    HandleSnack(`O usuário: ${name} foi atualizado`, true, 'success', '#228B22')
                else {
                    if (data.errors) {
                        HandleSnack(`${data.errors[0]}`, true, 'warning', '#FA8072')
                    }
                    if (data.email) {
                        let emailError = "";
                        data.email.map((msg) => (
                            emailError = emailError + msg + " e "
                        ))
                        setErrorInEmail({
                            error: true,
                            message: emailError
                        })
                    }
                    if (data.terms_of_service) {
                        let termsError = "termos de serviço ";
                        data.terms_of_service.map((msg) => (
                            termsError = termsError + msg + " e "
                        ))
                        HandleSnack(termsError, true, 'warning', '#FFC125')
                    }
                }
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                setIsLoading(false)
            }
        )
    }

    const onCreate = () => {
        setIsLoading(true)
        let userRolesID = [];
        const api = "/users"
        userRoles.map((role) => (
            userRolesID.push(role.id)
        ))
        const body = {
            "user": {
                "name": name,
                "email": email,
                "description": description,
                "role_ids": userRolesID,
                "password": pass,
                "password_confirmation": confirmPass,
                "terms_of_service": switchState
            }
        }
        postRequest(
            api,
            body,
            (data) => {
                setIsLoading(false)
                if (data.id) {
                    HandleSnack(`O usuário: ${name} foi criado`, true, 'success', '#228B22')
                    history.push("/admin/usersList")
                }
                else {
                    if (data.errors) {
                        HandleSnack(`${data.errors[0]}`, true, 'warning', '#FA8072')
                    }
                    if (data.password) {
                        let passError = "";
                        data.password.map((msg) => (
                            passError = passError + msg + " e "
                        ))
                        setErrorInPass({
                            error: true,
                            message: passError
                        })
                    }
                    if (data.email) {
                        let emailError = "";
                        data.email.map((msg) => (
                            emailError = emailError + msg + " e "
                        ))
                        setErrorInEmail({
                            error: true,
                            message: emailError
                        })
                    }
                    if (data.terms_of_service) {
                        let termsError = "";
                        data.terms_of_service.map((msg) => (
                            termsError = termsError + msg + " e "
                        ))
                        HandleSnack(termsError, true, 'warning', '#FFC125')
                    }
                    if (data.password_confirmation) {
                        let confirmError = "";
                        data.password_confirmation.map((msg) => (
                            confirmError = confirmError + msg + " e "
                        ))
                        setErrorInConfirmPass({
                            error: true,
                            message: confirmError
                        })
                    }
                }
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                setIsLoading(false)
            }
        )
    }

    const handleChange = (id, value) => {
        const currUserRoles = [...userRoles];
        currUserRoles.push(
            {
                id: id,
                name: value,
            }
        )
        setRolesList((chips) => chips.filter((chip) => chip.id !== id));
        setUserRoles(currUserRoles);
    };

    useEffect(() => {
        if (id !== "-1") {
            getRequest(
                GetAData("users", id),
                (data, header) => {
                    const currRolesList = [...rolesList];
                    const auxiliarRolesId = [
                        1, 2, 3, 4, 5, 6, 7, 9, 10, 11
                    ]
                    for (let i = 0; i < data.roles.length; i++) {
                        const rol = data.roles[i];
                        const index = auxiliarRolesId.indexOf(rol.id);
                        currRolesList.splice(index, 1);
                        auxiliarRolesId.splice(index, 1);
                    }
                    setRolesList(currRolesList);
                    setName(data.name)
                    setEmail(data.email)
                    setDescription(data.description)
                    setUserRoles(data.roles)
                    setSwitchState(data.terms_accepted_at ? true : false)
                    setIsLoaded(true);
                },
                (error) => {
                    setIsLoaded(true);
                    setError(error);
                }
            )
        }
    }, []);

    if (error) {
        return <div> Houve um erro... </div>
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else if (CheckUserPermission()) {
        return (
            <Card>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container direction='row' justify='space-between' alignItems="center">
                        <Grid item>
                            <Typography variant='h4'>
                                {name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Grid container direction="row">
                                <Grid item>
                                    <Link style={{ textDecoration: 'none' }} to={'/admin/usersList'}>
                                        <Button
                                            startIcon={<ListRoundedIcon />}
                                            variant='outlined'
                                            color='primary'
                                        >
                                            Listar
                                        </Button>
                                    </Link>
                                </Grid>
                                {
                                    id !== "-1" ?
                                        <Grid item>
                                            <Button
                                                startIcon={<DeleteRoundedIcon />}
                                                color="secondary"
                                                variant="outlined"
                                                onClick={() => { deleteUser(id) }}
                                            >
                                                Deletar
                                            </Button>
                                        </Grid> : null
                                }
                            </Grid>
                        </Grid>
                    </Grid>
                    <div style={{ height: '1em' }}></div>
                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        {fields.map((field, index) => (
                            field.value !== "-1" ?
                                <TextField
                                    key={index}
                                    required={field.required}
                                    disabled={field.default}
                                    error={field.error}
                                    helperText={field.error ? field.errorMessage : ''}
                                    style={{ width: '250px', marginBottom: '1em' }}
                                    label={field.label}
                                    value={field.value}
                                    onChange={field.onChange}
                                    type="search"
                                    multiline={true}
                                /> : null
                        ))}
                        {userRoles.map((data) => {
                            return (
                                <li key={data.id} style={{ listStyleType: "none" }}>
                                    <Chip
                                        label={data.name}
                                        onDelete={data === 'React' ? undefined : handleDelete(data)}
                                        className={classes.chip}
                                    />
                                </li>
                            );
                        })}
                        <TextField
                            id="standard-select-currency"
                            select
                            label="Permissões"
                            style={{ width: '250px', marginBottom: '1em' }}
                        >
                            {rolesList.map((option) => (
                                <MenuItem key={option.id} value={option.value} onClick={() => { handleChange(option.id, option.value) }}>
                                    {option.value}
                                </MenuItem>
                            ))}
                        </TextField>
                        <FormControlLabel
                            control={
                                <Switch
                                    checked={switchState}
                                    onChange={() => { setSwitchState(!switchState) }}
                                    name="checkedB"
                                    color="primary"
                                />
                            }
                            label="Concorda com os termos de serviço?"
                        />
                    </form>
                </CardContent>
                <CardAction>
                    {
                        id !== "-1" ?
                            <Button
                                onClick={() => {
                                    onSubmit();
                                }}
                                variant="contained"
                                color="primary"
                                disabled={isLoading}
                                startIcon={isLoading ? null : <SaveIcon />}
                            >
                                {
                                    isLoading ? <CircularProgress size={24} /> : 'Salvar'
                                }
                            </Button>
                            :
                            <Button
                                onClick={() => {
                                    onCreate();
                                }}
                                variant="contained"
                                color="primary"
                                disabled={isLoading}
                                startIcon={isLoading ? null : <AddRoundedIcon />}
                            >
                                {
                                    isLoading ? <CircularProgress size={24} /> : 'Criar'
                                }
                            </Button>
                    }
                </CardAction>
            </Card>
        )
    } else return <Unauthorized />
}

export default EditUser;
